@@ -4,7 +4,7 @@ from click_didyoumean import DYMGroup
 
 @click.group(cls=DYMGroup)
 def cli():
-    pass
+    """Just a simple provisioner"""
 
 
 def start():
